{#- OZI 0.1
# ozi/templates/project.name/meson.build.j2
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-#}
{{ '# OZI ' ~ spec.version }} meson.build:source_files,source_children
{%- for line in project.copyright_head.split('\n') %}
{{ '# ' ~ line }}
{%- endfor %}
{{ '# DO NOT EDIT BY HAND - This file was generated for use with OZI tools.' }}
{{ '# ozi-fix -> meson rewrite is the standard editing pipeline.' }}
source_files = ['__init__.py', 'py.typed']
foreach file: source_files
    fs.copyfile(file)
    if not meson.is_subproject() or get_option('install-subprojects').enabled()
        python.install_sources(file, subdir: '{{ project.name|underscorify }}')
    endif
endforeach
source_children = []
foreach child: source_children
    subdir(child)
endforeach
